import { Component, ViewChild, DoCheck } from "@angular/core";

import { ResultElement } from "jalhyd";

import { SectionCanvasComponent } from "../section-canvas/section-canvas.component";
import { SectionResults } from "../../results/section-results";
import { ApplicationSetupService } from "../../services/app-setup/app-setup.service";
import { CalculatorResults } from "../../results/calculator-results";
import { ResultsComponent } from "../fixedvar-results/results.component";
import { I18nService } from "../../services/internationalisation/internationalisation.service";

@Component({
    selector: "section-results",
    templateUrl: "./section-results.component.html",
    styleUrls: [
        "./section-results.component.scss"
    ]
})
export class SectionResultsComponent extends ResultsComponent implements DoCheck {

    constructor(
        private appSetupService: ApplicationSetupService,
        private intlService: I18nService
    ) {
        super();
    }

    public set results(rs: CalculatorResults[]) {
        this._resultElement = undefined;
        this._results = undefined;
        if (rs) {
            for (const r of rs) {
                if (r instanceof SectionResults) {
                    this._results = r;
                }
            }
        }
        this.updateView();
    }

    /** détermine s'il y a des résultats de section "fixes" (pas de paramètre varié, graphique de la section) */
    public get hasResults(): boolean {
        return this._results && this._results.hasResults;
    }

    public get resultElement() {
        return this._resultElement;
    }

    private static labelColors: { [key: string]: any; } = {
        "Hs": { r: 255, g: 0, b: 0 },
        "Hsc": { r: 0, g: 0, b: 255 },
        "Yn": { r: 16, g: 128, b: 16 },
        "Yf": { r: 128, g: 128, b: 128 },
        "Yt": { r: 0, g: 0, b: 0 },
        "Yc": { r: 255, g: 128, b: 0 },
        "Yco": { r: 255, g: 0, b: 255 },
    };

    /**  résultats non mis en forme */
    private _results: SectionResults;

    /**
     * ResultElement mis en forme (symboles des variables traduits)
     */
    private _resultElement: ResultElement;

    private _doUpdate = false;

    @ViewChild(SectionCanvasComponent, { static: false })
    private _sectionCanvas: SectionCanvasComponent;

    public updateView() {
        if (this._sectionCanvas) {
            this._sectionCanvas.reset();
        }

        if (this._results) {
            this._doUpdate = this._results.hasResults;
        }
    }

    /**
     * appelé pour gérer les changements non détectés par Angular
     */
    public ngDoCheck() {
        if (this._doUpdate) {
            this._doUpdate = !this.updateResults();
        }
    }

    private isSectionLevel(s: string) {
        for (const k in SectionResultsComponent.labelColors) {
            if (k === s) {
                return true;
            }
        }
        return false;
    }

    private updateResults() {
        if (this._results && this._sectionCanvas) {
            const nDigits = this.appSetupService.displayDigits;

            this._resultElement = new ResultElement();

            // traduction des symboles des variables calculées
            for (const k in this._results.result.extraResults) {
                const lbl = k.toUpperCase();
                const er = this._results.result.getExtraResult(k);
                this._resultElement.addExtraResult(lbl, er);

                if (this.isSectionLevel(k)) {
                    this._sectionCanvas.addLevel(er, k + " = " + er.toFixed(nDigits), SectionResultsComponent.labelColors[k]);
                }
            }

            this._sectionCanvas.section = this._results.section;
            return true;
        }
        return false;
    }

    private getResultClass(i: number) {
        // tslint:disable-next-line:no-bitwise
        return "result_id_" + String(i & 1);
    }

    public exportAsImage(element: HTMLDivElement) {
        const canvas: HTMLCanvasElement = element.querySelector("canvas");
        canvas.toBlob((blob) => {
            saveAs(blob, "chart.png");
        }); // defaults to image/png
    }

    public get uitextExportImageTitle() {
        return this.intlService.localizeText("INFO_GRAPH_BUTTON_TITLE_EXPORT_IMAGE");
    }

    public get uitextEnterFSTitle() {
        return this.intlService.localizeText("INFO_GRAPH_BUTTON_TITLE_ENTER_FS");
    }

    public get uitextExitFSTitle() {
        return this.intlService.localizeText("INFO_GRAPH_BUTTON_TITLE_EXIT_FS");
    }
}
