import { Nub } from "jalhyd";

import { FormCompute } from "./form-compute";
import { NgParameter, ParamRadioConfig } from "../ngparam";
import { FormResultFixedVar } from "./form-result-fixedvar";
import { FormulaireDefinition } from "./form-definition";

export class FormComputeFixedVar extends FormCompute {
    constructor(formBase: FormulaireDefinition, formResult: FormResultFixedVar) {
        super(formBase, formResult);
    }

    protected get formResult(): FormResultFixedVar {
        return this._formResult as FormResultFixedVar;
    }

    private getVariatedParameters(): NgParameter[] {
        let res: NgParameter[] = [];
        // find variated local parameters
        res = this._formBase.getDisplayedParamListFromState(ParamRadioConfig.VAR);
        // add variated linked parameters
        const pms = this._formBase.getDisplayedParamListFromState(ParamRadioConfig.LINK);
        for (const p of pms) {
            if (p.paramDefinition.hasMultipleValues) {
                res.push(p);
            }
        }
        return res;
    }

    private getComputedParameter(): NgParameter {
        return this._formBase.getDisplayedParamFromState(ParamRadioConfig.CAL);
    }

    protected compute() {
        this.runNubCalc(this._formBase.currentNub);
        this.reaffectResultComponents();
    }

    protected reaffectResultComponents() {
        const nub: Nub = this._formBase.currentNub;
        const computedParam: NgParameter = this.getComputedParameter();
        this.formResult.resetResults(); // to avoid adding fixed parameters more than once (see below)
        this.formResult.addFixedParameters();
        const varParams: NgParameter[] = this.getVariatedParameters();

        if (varParams.length === 0) {
            // pas de paramètre à varier
            this.formResult.fixedResults.result = nub.result;
            this.formResult.fixedResults.calculatedParameter = computedParam;
        } else {
            // il y a un paramètre à varier
            this.formResult.varResults.variatedParameters = varParams;
            this.formResult.varResults.calculatedParameter = computedParam;

            this.formResult.varResults.result = nub.result;
            this.formResult.varResults.update(false);
        }
    }
}
