import { NgParameter, ParamRadioConfig } from "../ngparam";
import { FormulaireDefinition } from "./form-definition";
import { FieldSet } from "../fieldset";

export class FormDefSection {
    /**
     * id de l'élément configurant le type de section
     */
    private _sectionSourceId: string;

    private _formBase: FormulaireDefinition;

    constructor(base: FormulaireDefinition) {
        this._formBase = base;
    }

    private get hasSectionNodeTypeSource(): boolean {
        return this._sectionSourceId !== undefined;
    }

    public getSectionVariatedParameters(): NgParameter[] {
        return this._formBase.getDisplayedParamListFromState(ParamRadioConfig.VAR);
    }

    public getSectionComputedParam(): { symbol: string, label: string } {
        const symbol = this._formBase.getSelectedValue("select_target");
        const label = this._formBase.getSelectedLabel("select_target");
        return { symbol, label };
    }

    public parseOptions(json: {}) {
        // id de l'élément configurant le type de section
        this._sectionSourceId = this._formBase.getOption(json, "sectionSourceId");
    }

    /**
     * appelé après la création d'un FieldSet
     * @param fs nouveau FieldSet
     */
    public afterParseFieldset(fs: FieldSet) {
        if (this.hasSectionNodeTypeSource) { // s'il existe un menu de choix de section dans le module de calcul
            const sel = fs.getFormulaireNodeById(this._sectionSourceId);
            if (sel) {
                // on abonne le formulaire aux propriétés du FieldSet pour MAJ du nub, reset du formulaire, ...
                fs.properties.addObserver(this._formBase);
            }
        }
    }
}
