import { GraphType } from "./graph-type";

/**
 * Une interface pour nourrir un ResultsGraphComponent
 */
export interface PlottableData {

    graphType: GraphType;
    chartX: string;
    chartY: string;

    /**
     * Returns the label to display, for an element of getAvailableChartAxis()
     * (usually a variable symbol like "Q", "Z1"…)
     * @param symbol parameter / result symbol (ex: "Q")
     */
    getChartAxisLabel(symbol: string): string;

    /**
     * Returns the translated name of the given symbol (usually an extraResult)
     * if available, with its unit, but without the symbol itself
     */
    expandLabelFromSymbol(symbol: string): string;

    /**
     * Returns a list of plottable parameters / result elements, that can be defined
     * as X or Y chart axis
     */
    getAvailableChartAxis(): string[];

    /**
     * Returns the series of values for the required variated parameter / result element
     * @param symbol parameter / result symbol (ex: "Q")
     */
    getValuesSeries(symbol: string): any[];

    /**
     * Returns the list of variating parameters
     * (used by tooltip functions)
     */
    getVariatingParametersSymbols(): string[];
}
